"""Google OR-Tools CP-SAT solver implementation.

This package provides a complete constraint solver implementation using
Google OR-Tools CP-SAT engine.
"""

from chuk_mcp_solver.solver.ortools.solver import ORToolsSolver

__all__ = ["ORToolsSolver"]
